<#

.SYNOPSIS 
Creates a Group Policy object which configures computers for use with Citrix App Studio.

.DESCRIPTION 
Citrix App Studio requires some settings to be configured on the computers which it
manages; for example, PowerShell remoting must be enabled, the PowerShell script execution
policy must be set, etc.  For a detailed description of all of the settings required,
consult the Citrix App Studio Administrator guide.

This script creates a GPO with all required settings enabled as per the Administrator guide
requirements, and optionally links the GPO to one or more OUs in your domain.

.PARAMETER PolicyName
Name of the GPO to create.  If the GPO already exists, you will be prompted to overwrite or
exit, unless using -Force.  Defaults to "CtxCloudAppManagement".

.PARAMETER LinkTo
List of OUs to link the GPO to.  If not specified, the GPO will be created but will not be
linked to any OU.  The OUs must be specified in canonical format, without the domain. For
example, "EMEA/Resellers/Customers" would refer to an OU in the current machine's domain,
with a DN such as "OU=Customers,OU=Resellers,OU=EMEA,DC=domain,DC=com".  The root of the
domain may be specified as "/".

.PARAMETER Force
If specified, the GPO will be deleted if it already exists, without prompting.

.PARAMETER Passthru
If specified, the newly created GPO will be output when the script exits.


.EXAMPLE
New-CtxAppStudioGPO.ps1

Creates a GPO named "CtxCloudAppManagement" in the domain.  The GPO is not linked.

.EXAMPLE
New-CtxAppStudioGPO.ps1 -PolicyName MyPolicy

Creates a GPO named "MyPolicy" in the domain.  The GPO is not linked.

.EXAMPLE
New-CtxAppStudioGPO.ps1 -LinkTo CloudAppManagement,Customers

Creates a GPO named "CtxCloudAppManagement" in the domain.  The GPO is linked to the
"CloudAppManagement" and "Customers" OUs.

.EXAMPLE
New-CtxAppStudioGPO.ps1 -LinkTo /

Creates a GPO named "CtxCloudAppManagement" in the domain.  The GPO is linked to the root
OU of the domain.

.NOTES
    KEYWORDS: powershell, terminal services, CSP, Cloud App Studio
    REQUIRES: 
    1. GroupPolicy Module
    2. Windows 2008 R2 or Windows 7
    3. PowerShell Version 2.0
    4. Computer joined to a Domain

	Copyright (c) 2012 Citrix Systems, Inc. All rights reserved.

.LINK
     http://community.citrix.com/p/csp
#>

[CmdletBinding(SupportsShouldProcess=$true)]
Param(
  [Parameter(Position=0, HelpMessage="Name of the GPO to create.")]
  [Alias("Name")]
  [string]$PolicyName = 'CtxCloudAppManagement',
  [Parameter(Position=1, HelpMessage="List of OUs to link the GPO to.")]
  [string[]]$LinkTo,
  [Parameter(HelpMessage="Deletes GPO if it exists without confirmation.")]
  [switch]$Force,
  [Parameter(HelpMessage="Output the GPO when the script completes.")]
  [switch]$Passthru
)

Set-StrictMode -Version Latest

Import-Module ServerManager
$gpmcFeature = Get-WindowsFeature GPMC
if (!$gpmcFeature.Installed)
{
  if ($pscmdlet.ShouldProcess($gpmcFeature.DisplayName, "Install Windows Feature"))
  {
    if ($Force -or $pscmdlet.ShouldContinue("Install $($gpmcFeature.DisplayName)?", "This script requires the '$($gpmcFeature.DisplayName)' Windows Feature to be installed."))
    {
      Add-WindowsFeature GPMC
    }
    else
    {
      Throw "$($gpmcFeature.DisplayName) is not installed.  Aborting."
    }
  }
}

Import-Module GroupPolicy

Function SplitOU
{
    param($ou)
    if (!$ou) { return }
    
    $escaped = $false
    $start = 0
    
    for ($end = 0; $end -lt $ou.Length; $end++)
    {
        if ($ou[$end] -eq '\')
        {
            $escaped = !$escaped
            continue
        }
        if (!$escaped -and $ou[$end] -eq '/')
        {
            $len = $end - $start
            if ($len -gt 0)
            {
                $ou.Substring($start, $end - $start)
            }    
            $start = $end + 1
        }
        $escaped = $false
    }
    if ($start -lt $ou.Length)
    {
        $ou.Substring($start)
    }    
}

Function EscapeDN
{
    param($str)
    $str -replace '(\\|\<|\>|\,|\#|\;|\"|\=|\#|\+)', '\$&'
}

$domain = [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain().name

Write-Host "Create policy '$PolicyName' in domain '$domain'" -Fore Cyan

$existing = $null
$existing = Get-Gpo $PolicyName -ErrorAction SilentlyContinue
if ($existing -ne $null)
{
  if ($pscmdlet.ShouldProcess("$PolicyName", "Delete GPO"))
  {
    if ($Force -or $pscmdlet.ShouldContinue("Delete existing GPO?", "GPO '$PolicyName' already exists"))
    {
      $existing | Remove-GPO
    }
    else
    {
      Throw "Aborted."
    }
  }
}

$gpo = $null
if ($pscmdlet.ShouldProcess("$PolicyName", "Create GPO"))
{
  $gpo = New-GPO $PolicyName
}
else
{
  # Give it a try with an existing GPO if one exists.
  # You can get here if you use -Confirm and answer No to the first two questions.
  # You still get a chance to answer Yes to the last two questions, which would update
  # an existing policy.
  $gpo = Get-Gpo $PolicyName -ErrorAction SilentlyContinue
  if ($gpo -eq $null)
  {
    Throw "Cannot continue; GPO does not exist."
  }
}

if ($pscmdlet.ShouldProcess("$PolicyName", "Write GPO Registry Settings"))
{
    $Key = 'HKLM\Software\Policies\Microsoft\Windows\PowerShell'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName EnableScripts       -Type DWord  -Value 1            |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName ExecutionPolicy     -Type String -Value RemoteSigned |Out-Null

    $Key = 'HKLM\Software\Policies\Microsoft\Windows\WinRM\Client'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName TrustedHosts        -Type DWord  -Value 1            |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName TrustedHostsList    -Type String -Value '*'          |Out-Null

    $Key = 'HKLM\Software\Policies\Microsoft\Windows\WinRM\Service'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName AllowAutoConfig     -Type DWord  -Value 1            |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName IPv4Filter          -Type String -Value '*'          |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName IPv6Filter          -Type String -Value '*'          |Out-Null

    $Key = 'HKLM\Software\Policies\Microsoft\Windows\WinRM\Service\WinRS'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName MaxMemoryPerShellMB -Type DWord  -Value 1000         |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName MaxShellsPerUser    -Type DWord  -Value 0            |Out-Null

    $Key = 'HKLM\Software\Policies\Microsoft\WindowsFirewall'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName PolicyVersion       -Type DWord  -Value 522          |Out-Null

    $Key = 'HKLM\Software\Policies\Microsoft\WindowsFirewall\FirewallRules'
    $gpo|Set-GPRegistryValue -Key $Key -ValueName WINRM-HTTP-Compat-In-TCP -Type String -Value 'v2.10|Action=Allow|Active=TRUE|Dir=In|Protocol=6|LPort=80|App=System|Name=@FirewallAPI.dll,-35001|Desc=@FirewallAPI.dll,-35002|EmbedCtxt=@FirewallAPI.dll,-30252|'   |Out-Null
    $gpo|Set-GPRegistryValue -Key $Key -ValueName WINRM-HTTP-In-TCP        -Type String -Value 'v2.10|Action=Allow|Active=TRUE|Dir=In|Protocol=6|LPort=5985|App=System|Name=@FirewallAPI.dll,-30253|Desc=@FirewallAPI.dll,-30256|EmbedCtxt=@FirewallAPI.dll,-30252|' |Out-Null
}

if ($pscmdlet.ShouldProcess("$PolicyName", "Write GPO Registry Preference Settings"))
{
    $Key = 'HKLM\SOFTWARE\Citrix\CloudAppManagement\Agent'
    $gpo|Set-GPPrefRegistryValue -Context Computer -Key HKLM\SOFTWARE\Citrix\CloudAppManagement\Agent -Action Update -Type DWord -ValueName SyncIntervalSeconds -Value 15 -Order 1 | Out-Null
}

if ($pscmdlet.ShouldProcess("$PolicyName", "Write GPO Service Settings"))
{
    # Manually add the inf setting to configure WinRM to start automatically
    # NOTE: this setting does not show up in the policy settings report.
    # I can find no way to fix this problem.  Perhaps if it was exported and
    # then imported?  But then I might need to worry about migration tables...

    $inf = @'
[Unicode]
Unicode=yes
[Version]
signature="$CHICAGO$"
Revision=1
[Service General Setting]
"WinRM",2,""
'@
    $path = "\\$domain\sysvol\$($domain)\Policies\{$($gpo.Id)}\Machine\"
    $path += "Microsoft\Windows NT\SecEdit"

    $file = (Join-Path $path 'GptTmpl.inf')
    $conflict = $false;
    if (Test-Path $file)
    {
      $conflict = $true;
      if ($pscmdlet.ShouldProcess("$PolicyName", "Overwrite existing GPO Service Settings"))
      {
        if ($Force -or $pscmdlet.ShouldContinue("Overwrite GPO service security settings?", "GPO '$PolicyName' has existing service security settings"))
        {
          $conflict = $false;
        } 
      }
    }
    if ($conflict)
    {
       Write-Error "Not writing GPO service security settings."
    }
    else
    {
      if (!(Test-Path $path)) {
        md $path | Out-Null
      }
      $inf | Out-File $file | Out-Null
    }
}

Write-Host "Created GPO '$PolicyName'." -Fore Cyan

$LinkTo | ?{$_} |% {
  $target = $_
  $split = @(SplitOU $target)
  [array]::Reverse($split)
  $ouparts = $split |% {"OU=$(EscapeDN $_)"}
  $domparts = $domain.Split(".") |% {"DC=$_"}
  $allparts = (@($ouparts)+@($domparts))|?{$_}
  $dn = [string]::Join(",", $allparts)

  if ($pscmdlet.ShouldProcess("$dn", "Link GPO '$PolicyName' to OU"))
  {
      $gpo|New-GPLink -Target $dn | Out-Null
  }
}

if ($Passthru)
{
  $gpo
}

# SIG # Begin signature block
# MIIXoAYJKoZIhvcNAQcCoIIXkTCCF40CAQExDjAMBggqhkiG9w0CBQUAMGgGCisG
# AQQBgjcCAQSgWjBYMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYp
# fvNRAgEAAgEAAgEAAgEAAgEAMCAwDAYIKoZIhvcNAgUFAAQQxbiD3ubD+cQ6Wakk
# Lqjxs6CCEvYwggN6MIICYqADAgECAhA4Jdf6+GGvnvSQ5ya11lrVMA0GCSqGSIb3
# DQEBBQUAMFMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjEr
# MCkGA1UEAxMiVmVyaVNpZ24gVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQTAeFw0w
# NzA2MTUwMDAwMDBaFw0xMjA2MTQyMzU5NTlaMFwxCzAJBgNVBAYTAlVTMRcwFQYD
# VQQKEw5WZXJpU2lnbiwgSW5jLjE0MDIGA1UEAxMrVmVyaVNpZ24gVGltZSBTdGFt
# cGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHMjCBnzANBgkqhkiG9w0BAQEFAAOBjQAw
# gYkCgYEAxLXyUhW8iIZgKRZKWy9LkWuHkfM1VFg16tE2XmJNUlE0ccJ7Zh2JyN0q
# xGoK9jfZmHSR9pKusLV2lvGpSmNFRy5rC5JOSyuM7lhKi9QH5Bos+IKqWNnNQvMt
# wHXejavHjh2abEwIlR7e2+9n4XLCScKeYDzh4r4Wo2N4aRR7rS0CAwEAAaOBxDCB
# wTA0BggrBgEFBQcBAQQoMCYwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLnZlcmlz
# aWduLmNvbTAMBgNVHRMBAf8EAjAAMDMGA1UdHwQsMCowKKAmoCSGImh0dHA6Ly9j
# cmwudmVyaXNpZ24uY29tL3Rzcy1jYS5jcmwwFgYDVR0lAQH/BAwwCgYIKwYBBQUH
# AwgwDgYDVR0PAQH/BAQDAgbAMB4GA1UdEQQXMBWkEzARMQ8wDQYDVQQDEwZUU0Ex
# LTIwDQYJKoZIhvcNAQEFBQADggEBAFDFS8gkgN/kDSTC3hqxoQKhpoItDIMVgTcK
# gg4ssFoXYbXYBf6I2/GRkbNWGkCm65K+ODmwdTZ0OphP5De6mYnKlUIdsLnHoI1X
# 4PrVZARCNU4B0TOiF8hNqifH8uGGTAI4TYN4xvxT4OvgBofdpJaeXgyY4qW+v4KF
# w2Dh360o2MelS2Taxxtbvaw5CNU4IqEziy+Kmuu8ByE/REEJB7VlHCS8SNNEgOuh
# z8kCtBTPVMcWo4Bc+Xk+XXJ9iBeeLEOiylPOfT32Kjq4T5QApW0Kg135XlP0GLNX
# D3DD+/WtlaAOF97EFoBgyQ8rboYE8ev0eCfRBcXuNFteuUky8jMwggPEMIIDLaAD
# AgECAhBHvxmV341SRkP3221IDTGkMA0GCSqGSIb3DQEBBQUAMIGLMQswCQYDVQQG
# EwJaQTEVMBMGA1UECBMMV2VzdGVybiBDYXBlMRQwEgYDVQQHEwtEdXJiYW52aWxs
# ZTEPMA0GA1UEChMGVGhhd3RlMR0wGwYDVQQLExRUaGF3dGUgQ2VydGlmaWNhdGlv
# bjEfMB0GA1UEAxMWVGhhd3RlIFRpbWVzdGFtcGluZyBDQTAeFw0wMzEyMDQwMDAw
# MDBaFw0xMzEyMDMyMzU5NTlaMFMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5WZXJp
# U2lnbiwgSW5jLjErMCkGA1UEAxMiVmVyaVNpZ24gVGltZSBTdGFtcGluZyBTZXJ2
# aWNlcyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKnKsqTMzSCv
# Cn2JrId18LRO8d/BD79nYb2jZBzau/nKM6uEMIlYfozba902ng+/0ex48nemfm88
# v5OvDbpo9GyUyr1SLatIPfW21V1fGwKf+i9rHqT3o5qmGsgC4X9MUuMOYOxAHH65
# Dd4/x7Tfh71femoxLgOZgROoRyDOMXMNVy3NeDQzlRKZErneaC+q5uPCiowqw4sh
# h2a9g1hXb3W/PKomh13KEBU8n4TqVMEKbsT+xUrduQcRlyJ82z4n0R547J8xyfHm
# IhnbxLNHQ5oaX6AekORe9e588X2rYgGP9U0L3tAiVqiVza6Idq7uug3z5E3ZoPto
# oK4UO7OHwbsCAwEAAaOB2zCB2DA0BggrBgEFBQcBAQQoMCYwJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLnZlcmlzaWduLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEAMEEG
# A1UdHwQ6MDgwNqA0oDKGMGh0dHA6Ly9jcmwudmVyaXNpZ24uY29tL1RoYXd0ZVRp
# bWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNVHQ8BAf8E
# BAMCAQYwJAYDVR0RBB0wG6QZMBcxFTATBgNVBAMTDFRTQTIwNDgtMS01MzANBgkq
# hkiG9w0BAQUFAAOBgQBKa/nqWMJEHDGJeZkrlr+CrAHWHEzNsIpYbt8IKaNeyMqT
# E+cEUg3vRycvADiw5MmTTprUImIV9z83IU9wMYDxiziHs+jolwD+z1WWTiTSqSdO
# eq63YUHzKs7nydle3bsrhT61nbXZ4Vf/vrTFfvXPDJ7wl/4r0ztSGxs4J/c/SjCC
# BZ4wggSGoAMCAQICEAWm+U5cQrfXZqjrZQWOZ6owDQYJKoZIhvcNAQEFBQAwgbQx
# CzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjEfMB0GA1UECxMW
# VmVyaVNpZ24gVHJ1c3QgTmV0d29yazE7MDkGA1UECxMyVGVybXMgb2YgdXNlIGF0
# IGh0dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9ycGEgKGMpMTAxLjAsBgNVBAMTJVZl
# cmlTaWduIENsYXNzIDMgQ29kZSBTaWduaW5nIDIwMTAgQ0EwHhcNMTExMTAzMDAw
# MDAwWhcNMTIxMTExMjM1OTU5WjCB4TELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0Zs
# b3JpZGExGDAWBgNVBAcTD0ZvcnQgTGF1ZGVyZGFsZTEdMBsGA1UEChQUQ2l0cml4
# IFN5c3RlbXMsIEluYy4xPjA8BgNVBAsTNURpZ2l0YWwgSUQgQ2xhc3MgMyAtIE1p
# Y3Jvc29mdCBTb2Z0d2FyZSBWYWxpZGF0aW9uIHYyMSgwJgYDVQQLFB9YZW5BcHAg
# RW5naW5lZXJpbmcgKFBvd2VyU2hlbGwpMR0wGwYDVQQDFBRDaXRyaXggU3lzdGVt
# cywgSW5jLjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALPW2tHJM6/A
# KZhSfwLLn1R011VKCaygHRgCYOkEBnzmJeozs3bIaU98g+ZFq60A5v6uA+YJO2bP
# ZE1+hlCTKwbOcKe1WGqCJ00Q5QqlBpMpyVB1MGPJ6sqNjcx6jBneq0yi2Fe3i5TG
# U7RnYqRZIbXRnwszpQulC6fvDa6C7/qF0MZ84bO7J16Utl7+rB5rKHG8sHjMngOy
# Mt5cblbjxluG3F8TkI3f+kljyFpIsY/OWpDxftdtSgwgq8Rs5K3P/0r0RRzOzo/e
# FLlySYC84JnI+ktFUYITu/hxNpSq2yg+1tpz60rrn77qa7MZuHUikq5ny34k5wsh
# EoCrH2AbKr0CAwEAAaOCAXswggF3MAkGA1UdEwQCMAAwDgYDVR0PAQH/BAQDAgeA
# MEAGA1UdHwQ5MDcwNaAzoDGGL2h0dHA6Ly9jc2MzLTIwMTAtY3JsLnZlcmlzaWdu
# LmNvbS9DU0MzLTIwMTAuY3JsMEQGA1UdIAQ9MDswOQYLYIZIAYb4RQEHFwMwKjAo
# BggrBgEFBQcCARYcaHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYTATBgNVHSUE
# DDAKBggrBgEFBQcDAzBxBggrBgEFBQcBAQRlMGMwJAYIKwYBBQUHMAGGGGh0dHA6
# Ly9vY3NwLnZlcmlzaWduLmNvbTA7BggrBgEFBQcwAoYvaHR0cDovL2NzYzMtMjAx
# MC1haWEudmVyaXNpZ24uY29tL0NTQzMtMjAxMC5jZXIwHwYDVR0jBBgwFoAUz5mp
# 6nsm9EvJjo/X8AUm7+PSp50wEQYJYIZIAYb4QgEBBAQDAgQQMBYGCisGAQQBgjcC
# ARsECDAGAQEAAQH/MA0GCSqGSIb3DQEBBQUAA4IBAQAhlDeBAomBO8B3VIvUGwF9
# dkDJ4DzHxG31eUVsnjjzPNWVzd41c2A6JPDrzOpZVhgRoCIdFY040nGE/2ll9S6S
# 1VF7T5g3DCwSFPKJivyRa5HH6PPpIsbKLflG1J6o65Lq3cXyyN7xWJgPCoE8vLbP
# f1LCkjkQOMA+BCH7Ua2gcJ7jb7Cs8qOuinKqNOo33q3ybGKOfcZTuPkbvI1tjgYG
# stCjul6onQaGNwFrfN8+hZE0W43lxrXTuFeJzWU+PO4PuwA4Nh1eK34ZhC1B7Fi8
# Myvff71BRmWRcE0ZQdS4Uv8mWiODles7qo9J9R07ez6xexfKqb+SXGEKEdbErGwx
# MIIGCjCCBPKgAwIBAgIQUgDlqiVW/BqG7ZbJ1EszxzANBgkqhkiG9w0BAQUFADCB
# yjELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQL
# ExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTowOAYDVQQLEzEoYykgMjAwNiBWZXJp
# U2lnbiwgSW5jLiAtIEZvciBhdXRob3JpemVkIHVzZSBvbmx5MUUwQwYDVQQDEzxW
# ZXJpU2lnbiBDbGFzcyAzIFB1YmxpYyBQcmltYXJ5IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzUwHhcNMTAwMjA4MDAwMDAwWhcNMjAwMjA3MjM1OTU5WjCBtDEL
# MAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZW
# ZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2UgYXQg
# aHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMlVmVy
# aVNpZ24gQ2xhc3MgMyBDb2RlIFNpZ25pbmcgMjAxMCBDQTCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBAPUjS16l14q7MunUV/fv5Mcmfq0ZmP6onX2U9jZr
# ENd1gTB/BGh/yyt1Hs0dCIzfaZSnN6Oce4DgmeHuN01fzjsU7obU0PUnNbwlCzin
# jGOdF6MIpauw+81qYoJM1SHaG9nx44Q7iipPhVuQAU/Jp3YQfycDfL6ufn3B3fkF
# vBtInGnnwKQ8PEEAPt+W5cXklHHWVQHHACZKQDy1oSapDKdtgI6QJXvPvz8c6y+W
# +uWHd8a1VrJ6O1QwUxvfYjT/HtH0WpMoheVMF05+W/2kk5l/383vpHXv7xX2R+f4
# GXLYLjQaprSnTH69u08MPVfxMNamNo7WgHbXGS6lzX40LYkCAwEAAaOCAf4wggH6
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwcAYDVR0gBGkwZzBlBgtghkgBhvhFAQcXAzBW
# MCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy52ZXJpc2lnbi5jb20vY3BzMCoGCCsG
# AQUFBwICMB4aHGh0dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9ycGEwDgYDVR0PAQH/
# BAQDAgEGMG0GCCsGAQUFBwEMBGEwX6FdoFswWTBXMFUWCWltYWdlL2dpZjAhMB8w
# BwYFKw4DAhoEFI/l0xqGrI2Oa8PPgGrUSBgsexkuMCUWI2h0dHA6Ly9sb2dvLnZl
# cmlzaWduLmNvbS92c2xvZ28uZ2lmMDQGA1UdHwQtMCswKaAnoCWGI2h0dHA6Ly9j
# cmwudmVyaXNpZ24uY29tL3BjYTMtZzUuY3JsMDQGCCsGAQUFBwEBBCgwJjAkBggr
# BgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNpZ24uY29tMB0GA1UdJQQWMBQGCCsG
# AQUFBwMCBggrBgEFBQcDAzAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVmVyaVNp
# Z25NUEtJLTItODAdBgNVHQ4EFgQUz5mp6nsm9EvJjo/X8AUm7+PSp50wHwYDVR0j
# BBgwFoAUf9Nlp8Ld7LvwMAnzQzn6Aq8zMTMwDQYJKoZIhvcNAQEFBQADggEBAFYi
# 5jSkxGHLSLkBrVaoZA/ZjJHEu8wM5a16oCJ/30c4Si1s0X9xGnzscKmx8E/kDwxT
# +hVe/nSYSSSFgSYckRRHsExjjLuhNNTGRegNhSZzA9CpjGRt3HGS5kUFYBVZUTn8
# WBRr/tSk7XlrCAxBcuc3IgYJviPpP0SaHulhncyxkFz8PdKNrEI9ZTbUtD1AKI+b
# EM8jJsxLIMuQH12MTDTKPNjlN9ZvpSC9NOsm2a4N58Wa96G0IZEzb4boWLslfHQO
# WP51G2M/zjF8m48blp7FU3aEW5ytkfqs7ZO6XcghU8KCU2OvEg1QhxEbPVRSloos
# nD2SGgiaBS7Hk6VIkdMxggQSMIIEDgIBATCByTCBtDELMAkGA1UEBhMCVVMxFzAV
# BgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVzdCBO
# ZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cudmVy
# aXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3MgMyBD
# b2RlIFNpZ25pbmcgMjAxMCBDQQIQBab5TlxCt9dmqOtlBY5nqjAMBggqhkiG9w0C
# BQUAoIGYMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMB8GCSqGSIb3DQEJBDESBBCPusgHIam3hOfvMlLwYQZD
# MDwGCisGAQQBgjcCAQwxLjAsoBiAFgBDAGkAdAByAGkAeAAgAEYAaQBsAGWhEIAO
# d3d3LmNpdHJpeC5jb20wDQYJKoZIhvcNAQEBBQAEggEATp29poBbpKZBw4R1b3kY
# wsJx9Vnziuod0VZf+KO7XxQvcFb9XgV46dxJ8b7RWIIajjs6EaBpqbZsDM/0pUkR
# CPf+2iWLyHh4xlLpBXihPlDJAGHDXqC2agf2TJuxI2AvTgmWQfGU183AHLAcYTPd
# 1w0R0CM5S7fQVOh/eFA95Ynnw6+2wZ6hQUmtU9ivviWYH7BbAtKGwc1IANsR7k4B
# /BRTcijpqOsdY9gQ5L9AZaz5qBD/Vp2aPyjWOa5ejz09LysDjEibre6DnI1pJfXY
# Z/vWyO0YF3ixxun4GM45P1LRS2uHDYGIvNWZXNkz+FdrQmMu8egE8g+TVv9O/Uto
# lKGCAX8wggF7BgkqhkiG9w0BCQYxggFsMIIBaAIBATBnMFMxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjErMCkGA1UEAxMiVmVyaVNpZ24gVGlt
# ZSBTdGFtcGluZyBTZXJ2aWNlcyBDQQIQOCXX+vhhr570kOcmtdZa1TAJBgUrDgMC
# GgUAoF0wGAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcN
# MTIwNDIzMTM0NjUyWjAjBgkqhkiG9w0BCQQxFgQUqPrxRKHScMe+WeJqvNHgg+sO
# PNowDQYJKoZIhvcNAQEBBQAEgYBVv1p/XvoiBAXattx5lMx09F3OXK4wUdR3sY9O
# PjtuqT1gdA3asVnFfXBWyAeg8SJx05jy90ieRomGFKtXaS2TSNT92BsMuEu+Tlxj
# dUe09/1AihPlJ1zrdUVn6nmXNrvConXKJgQKAsF/rPpaHSy3FfEiWAxNdg+7nBuN
# 9MKRkg==
# SIG # End signature block
